Attribute VB_Name = "Module1"
Option Explicit
'
' CALENDAR FUNCTION UTILITIES - Alpha Build - V0.001
'
' Written by Jay Phillips
November , 1998
' Using VB 5.0 Enterprise Edition running under Windows 98.
'
' A tutorial on programming basic calendar functions from scratch
' without using the date functions built into the VB5 language.
'
' It is expected that the user have at least a rudimentary of
' programming in VB, but it is not mandatory.  The functions are
' fully programmed to do the bulk of the computational work and
' ready to be applied in a project built around them.
'
' These functions are intended as freeware for anyone who wants to
' use them to construct a working program involving calendar functions.
' They go far beyond the usual calendar functions found in most computer
' languages and cover a computational span of many thousands of years.
' These modules were originally intended for application in astronomical
' computations, but can serve other technical purposes as well.
'
' NOTE:
' Because these are alpha modules, they contain no internal
' error checking for invalid input arguments.  The testing
' for valid parameters must be done prior to calling one of
' these alpha level functions for use.
'
' There is a special function included to test a date for
' validity before passing it to a function.
'
' Generally, if valid arguments are supplied, then a valid
' result should return.  If an invalid argument is passed,
' it could result in an unreliable result or possibly crash
' the program.
'

Public Function Is_Valid_Date(Dd_Mmm_Yyyy_BCAD, JG_Mode) As Boolean
'  This function determines whether or not a given date is valid.  If
'  it is valid, boolean (True) is returned, otherwise (False) is returned.
'
'  A typical example of valid date inputs are:
'  "3 jan 1300 BC"  or  "21 Dec 1905 AD"  or  "4 Jul 1776"
'
'  Version Alpha 0.001
'
'  Define the main function variables
   Dim Date_String, Cal_Mode As String
   Dim Day, Month, Year, BCAD, JD
   Dim Month_Abbreviation As String
    
'  Define random work variables
   Dim Q, Q1, Y
   Dim Pointer As Integer
    
' Version Alpha 0.001
' FUNCTION LEVEL 0 - No external function dependency.
 '
'  Read the (Cal_Mode) from the (JG_Mode).  This determines the calendar,
'  Julian or Gregorian, to which the computed JD number applies.
'  Only the 1st character of (JG_Mode) is used. If that character is "J"
'  then Julian mode is set, otherwise Gregorian "G" mode is set.
'  This test is NOT case sensitive.
   Cal_Mode = Left(Trim(UCase(JG_Mode)), 1)
If Cal_Mode <> "J" Then Cal_Mode = "G"
  
' ------------------------------------------------------
' Read calendar date argument to be tested for validity.

'   Strip all spaces from within date string.
    Date_String = Trim(UCase(Dd_Mmm_Yyyy_BCAD))
    Q = ""
For Pointer = 1 To Len(Date_String)
     Q1 = Mid(Date_String, Pointer, 1): If Q1 <> " " Then Q = Q & Q1
Next Pointer
     Date_String = Q
    
' Read the absolute value of the day of the month (1 to 31).
  Day = Trim(Abs(Val(Date_String)))

' Set (Pointer) to 1st character beyond the (Day) value.  This
' should be the 1st character in the 3 letter month name abbreviation
' in English. This simply means the 1st 3 letters in the month name
' and is NOT case sensitive.
  Pointer = InStr(1, Date_String, Day) + Len(Day)
   
' Now get the 3 letters of the month abbreviation and then update the (Pointer)
' to to aim at the following (Year) value.
  Month_Abbreviation = Mid(Date_String, Pointer, 3): Pointer = Pointer + 3
  
' Determine the number of the month corresponding to the three letter
' (Month_Abbreviation).  This abbreviation is simply the first three letters
' in the name of the month in English.  The returned value will range from
' 1=January to 12=December.  Zero is returned if no match is found
' for the abbreviation.  This test is NOT case sensitive.
  Month = Int(1 + ((InStr(1, UCase("JanFebMarAprMayJunJulAugSepOctNovDec"), _
          Month_Abbreviation) - 1) / 3))
  
'  Read the absolute value of the year (1 to 9999).  If no year is given or a
'  non-numeric value is given, then the year is invalid.
   Q = Trim(Mid(Date_String, Pointer, Len(Date_String)))
If Q = "" Then Is_Valid_Date = False: Exit Function
If Left(Q, 1) < "0" Or Left(Q, 1) > "9" Then Is_Valid_Date = False: Exit Function
   Year = Abs(Val(Q))
   
'  Check for BC or AD - If NOT a BC year, then assume AD.  This means that no
'  matter abbreviation follows the year, it will always be considered an AD
'  year unless BC is specifically indicated.  The year "123 QZ" would still be
'  read as the year "123 AD" instead of reporting an error.
If Right(Date_String, 2) = "BC" Then BCAD = "BC" Else BCAD = "AD"
If BCAD = "BC" Then Year = (1 - Year)

' At this point the separate numerical date elements should be collected
' and ready to be used for any subsequent calendar computations. They are:
' (Day: 1 to 31)  (Month: 1 to 12)  (Year: 1 to 9999 BC|AD)  (Cal_Mode: J|G)
' --------------------------------------------------------------------------
'  Now to test the given date for validity.  If the date is valid, then this
'  function will return boolean (True), if NOT valid, then (False).
'
'  Do a general 1st check of the day of the month number.  If the day number
'  is less than 1 or greater than 31, then it is an immediate error because
'  none of the months have more than 31 days and negative values are invalid.
'  Later, a further check will be done to account specific months and leap years.
If Int(Val(Day)) < 1 Or Int(Val(Day)) > 31 Then Is_Valid_Date = False: Exit Function

'  Check for an invalid month value outside the valid range of 1 to 12.
If Int(Val(Month)) < 1 Or Int(Val(Month)) > 12 Then Is_Valid_Date = False: Exit Function

'  Now check for a leap year.  If it is a leap year and February, then the
'  value of (Day) must not be greater than 29, or it is not a valid date.
If Year <= 0 Then Y = 1 - Year & " BC" Else Y = Year & " AD"
   GoSub TEST_LEAP_YEAR ' Q = Boolean (True) = Leap year
If Int(Val(Day)) > 29 And Q = True And Month = 2 Then Is_Valid_Date = False: Exit Function

'  Now check to see if the (Day) is in the correct range for the
'  specific given month.
If Int(Val(Day)) > Days_In_Month(Month_Abbreviation & " " & Y, Cal_Mode) Then
   Is_Valid_Date = False
   Exit Function
End If

' Drop through to here if given date has passed all validity checks so far.
  Is_Valid_Date = True
  Exit Function
  
'  Subroutine to check if a leap year - This is to make this a level zero function and
'  independent of external functions.
'
'  Determine if (Year) is leap on Julian calendar if "J" mode indicated,
'  otherwise, ignore this operation and solve for Gregorian calendar.
TEST_LEAP_YEAR:
   If Cal_Mode = "J" Then Q = ((Year Mod 4) = 0): GoTo LEAP_TEST_DONE
'  Determine if (Year) is leap on the Gregorian calendar.
If (Year Mod 100) = 0 Then Q = ((Year Mod 400) = 0): GoTo LEAP_TEST_DONE
   Q = ((Year Mod 4) = 0)
LEAP_TEST_DONE:
   Return

End Function

Public Function JD_Number_For(Dd_Mmm_Yyyy_BCAD, JG_Mode)
' Compute international, civil JD number for any given date
' on either the Julian or Gregorian calendars.
'
' Version Alpha 0.001
' FUNCTION LEVEL 0 - No external function dependency.
'
' This function assumes that all input arguments are valid.
' NO error checking is done.
'
'
'  Define the main function variables
   Dim Date_String, Cal_Mode As String
   Dim Day, Month, Year, BCAD, JD
   Dim Month_Abbreviation As String
    
'  Define random work variables
   Dim Q, Q1
   Dim Pointer As Integer
    
'  Read the (Cal_Mode) from the (JG_Mode).  This determines the calendar,
'  Julian or Gregorian, to which the computed JD number applies.
'  Only the 1st character of (JG_Mode) is used. If that character is "J"
'  then Julian "J" mode is set, otherwise Gregorian "G" mode is set.
'  This test is NOT case sensitive.
   Cal_Mode = Left(Trim(UCase(JG_Mode)), 1)
If Cal_Mode <> "J" Then Cal_Mode = "G"
  
' ------------------------------------------------
' Read calendar date (Assumed to be a valid date).
' No error tests are done in this alpha version.

'   Strip all spaces from within date string.
    Date_String = Trim(UCase(Dd_Mmm_Yyyy_BCAD))
    Q = ""
For Pointer = 1 To Len(Date_String)
     Q1 = Mid(Date_String, Pointer, 1): If Q1 <> " " Then Q = Q & Q1
Next Pointer
     Date_String = Q
    
' Read the absolute value of the day of the month (1 to 31).
  Day = Trim(Abs(Val(Date_String)))

' Set (Pointer) to 1st character beyond the (Day) value.  This
' should be the 1st character in the 3 letter month name abbreviation
' in English. This simply means the 1st 3 letters in the month name
' and is NOT case sensitive.
  Pointer = InStr(1, Date_String, Day) + Len(Day)
   
' Now get the 3 letters of the month abbreviation and then update the (Pointer)
' to to aim at the following (Year) value.
  Month_Abbreviation = Mid(Date_String, Pointer, 3): Pointer = Pointer + 3
  
' Determine the number of the month corresponding to the three letter
' (Month_Abbreviation).  This abbreviation is simply the first three letters
' in the name of the month in English.  The returned value will range from
' 1=January to 12=December.  Zero is returned if no match is found
' for the abbreviation.  This test is NOT case sensitive.
  Month = Int(1 + ((InStr(1, UCase("JanFebMarAprMayJunJulAugSepOctNovDec"), _
          Month_Abbreviation) - 1) / 3))
  
' Read the absolute value of the year (1 to 9999).
  Year = Abs(Val(Mid(Date_String, Pointer, Len(Date_String))))
  
'  Check for BC or AD - If NOT a BC year, then assume AD.
If Right(Date_String, 2) = "BC" Then BCAD = "BC" Else BCAD = "AD"
If BCAD = "BC" Then Year = (1 - Year)

' At this point the separate numerical date elements should be collected
' and ready to be used for any subsequent calendar computations. They are:
' (Day: 1 to 31)  (Month: 1 to 12)  (Year: 1 to 9999 BC|AD)  (Cal_Mode: J|G)
' --------------------------------------------------------------------------
'
'     Now compute (JD) number according to the original Julian calendar.
  Q = Int((14 - Month) / 12)
 JD = Day + Int(367 * (Month + (Q * 12) - 2) / 12) + Int(1461 * (Year + 4800 - Q) / 4) - 32113

'  Apply Julian (JD) correction to convert to (JD) according to the Gregorian calendar
'  if indicated by (Cal_Mode).
If Cal_Mode = "G" Then JD = JD - (Int(3 * Int((Year + 100 - Q) / 100) / 4) - 2)
    
'  Return the final JD value for output.
   JD_Number_For = JD
  
End Function

Public Function Is_Leap_Year(Year_BCAD, JG_Mode) As Boolean
' Returns boolean (True) if given year is leap, otherwise (False).
' Works for both Julian and Gregorian calendars and BC years.
' The (JG_Mode) is "J" for Julian or "G" for Gregorian.
'
' Version Alpha 0.001
' FUNCTION LEVEL 0 - No external function dependency.
'
' No error checks are done - Input is assumed to be valid.

' Define main function variables
  Dim Year_String, Year, BCAD, Cal_Mode

' Read value of (Year)
  Year_String = Trim(UCase(Year_BCAD))
  Year = Int(Abs(Val(Year_String)))
  
'  Check for BC or AD - If NOT a BC year, then assume AD.
If Right(Year_String, 2) = "BC" Then BCAD = "BC" Else BCAD = "AD"
If BCAD = "BC" Then Year = Abs(1 - Year)
 
'  Read the (Cal_Mode).  This determines the calendar, Julian or Gregorian,
'  to which the leap year applies.
'  Only the 1st character of (JG_Mode) is read. If that character is "J"
'  then Julian "J" mode is set, otherwise Gregorian "G" mode is set.
'  This test is NOT case sensitive.
   Cal_Mode = Left(Trim(UCase(JG_Mode)), 1)
If Cal_Mode <> "J" Then Cal_Mode = "G"
   
'  Determine if (Year) is leap on Julian calendar if "J" mode indicated,
'  otherwise, ignore this operation and solve for Gregorian calendar.
If Cal_Mode = "J" Then Is_Leap_Year = ((Year Mod 4) = 0): Exit Function

'  Determine if (Year) is leap on the Gregorian calendar.
If (Year Mod 100) = 0 Then Is_Leap_Year = ((Year Mod 400) = 0): Exit Function
   Is_Leap_Year = ((Year Mod 4) = 0)

End Function

Public Function JG_Calendar_Diff_For(Dd_Mmm_Yyyy_BCAD, JG_Mode)
' This function returns the difference in calendar days between
' the Julian and Gregorian calendars on any given date.  The
' value returned is relative to the date and mode arguments.
' This means that if the returned value is positive, then the
' other calendar is that many days ahead.  If negative, then
' the other calendar is behind by that many days.
' This difference is caused by the leap year rule differences
' between the Julian and Gregorian calendars.
' Over a period of 400 calendar years, the Julian calendar has 3
' more days than a 400 year period on the Gregorian calendar due
' to 3 leap years that are not recognized as leap years on the
' Gregorian calendar.  Over the centuries this difference
' between the calendars may accumulate to many days.  The most
' obvious sign of this would be the true date of spring, when the
' sun crosses the celestial equator in the northward direction,
' moving backwards through the calendar.  Eventually, seasonal
' spring would be starting in January if our modern Gregorian
' calendar wasn't now to used compensate for this drift to a much
' better extent than did the old Julian calendar.
'
' Version Alpha 0.001
' FUNCTION LEVEL: 0 - No external function dependency.
'
' No error checks are done - Input is assumed to be valid.

Dim Date_String, Cal_Mode As String
Dim Q, Q1
Dim Pointer As Integer
Dim Day, Month, Year, BCAD, JD

Dim Month_Abbreviations As String
Dim Month_Abbreviation As String
    Month_Abbreviations = "JanFebMarAprMayJunJulAugSepOctNovDec"
    
'  Read the (JG_Mode).  This determines the calendar, Julian or Gregorian,
'  to which the computed JD number applies.
'  Only the 1st character of (JG_Mode) is used. If that character is "J"
'  then Julian "J" mode is set, otherwise Gregorian "G" mode is set.
'  This test is NOT case sensitive.
   Cal_Mode = Left(Trim(UCase(JG_Mode)), 1)
If Cal_Mode <> "J" Then Cal_Mode = "G"
  
' ------------------------------------------------
' Read calendar date (Assumed to be a valid date).
' No error tests are done in this alpha version.

'   Strip all spaces from within date string.
    Date_String = Trim(UCase(Dd_Mmm_Yyyy_BCAD))
    Q = ""
For Pointer = 1 To Len(Date_String)
     Q1 = Mid(Date_String, Pointer, 1): If Q1 <> " " Then Q = Q & Q1
Next Pointer
     Date_String = Q
    
' Read the absolute value of the day of the month (1 to 31).
  Day = Trim(Abs(Val(Date_String)))

' Set (Pointer) to 1st character beyond the (Day) value.  This should
' be the 1st character in the 3 letter month name abbreviation.
  Pointer = InStr(1, Date_String, Day) + Len(Day)
   
' Now get the 3 letters of the month abbreviation and then update the (Pointer)
' to to aim at the following (Year) value.
  Month_Abbreviation = Mid(Date_String, Pointer, 3): Pointer = Pointer + 3
  
' Determine the number of the month corresponding to the three letter
' (Month_Abbreviation).  This abbreviation is simply the first three letters
' in the name of the month in English.  The returned value will range from
' 1=January to 12=December.  Zero is returned if no match is found
' for the abbreviation.  This test is NOT case sensitive.
  Month = Int(1 + ((InStr(1, UCase(Month_Abbreviations), _
          Month_Abbreviation) - 1) / 3))
  
' Read the absolute value of the year (1 to 9999).
  Year = Abs(Val(Mid(Date_String, Pointer, Len(Date_String))))
  
'  Check for BC or AD - If NOT a BC year, then assume AD.
If Right(Date_String, 2) = "BC" Then BCAD = "BC" Else BCAD = "AD"
If BCAD = "BC" Then Year = (1 - Year)

'  At this point the separate date elements should all be collected
'  and ready to be used for any subsequent calendar computations.
'  They are:  (Day) (Month) (Year) (BCAD) (Cal_Mode)
            
'  Compute Julian (JD) correction to convert to (JD) according to the Gregorian calendar.
   Q = Int(3 * Int((Year + 100 - Int((14 - Month) / 12)) / 100) / 4) - 2
    
'  Adjust sign for (Cal_Mode).  The value of (Q) will be the days to apply to the
'  current specified calendar date and mode to "correct" it to the other calendar.
   If Cal_Mode = "J" Then Q = -Q

'  Return the final calendar difference value for output.
   JG_Calendar_Diff_For = Q

End Function

Public Function Day_Of_Week_For(Dd_Mmm_Yyyy_BCAD, JG_Mode)
' Compute day of week for any given date on either the
' Julian or Gregorian calendar.
'
' Version Alpha 0.001
' FUNCTION LEVEL 0 - No external function dependency.
'
' This function assumes that all input arguments are valid.
' NO error checking is done.
'
'  Define the main function variables
   Dim Date_String, Cal_Mode As String
   Dim Day, Month, Year, BCAD, JD
   Dim Month_Abbreviations, Month_Abbreviation As String
       Month_Abbreviations = "JanFebMarAprMayJunJulAugSepOctNovDec"

'  Define random work variables
   Dim Q, Q1
   Dim Pointer As Integer
    
'  Read the (Cal_Mode) from the (JG_Mode).  This determines the calendar,
'  Julian or Gregorian, to which the computed JD number applies.
'  Only the 1st character of (JG_Mode) is used. If that character is "J"
'  then Julian "J" mode is set, otherwise Gregorian "G" mode is set.
'  This test is NOT case sensitive.
   Cal_Mode = Left(Trim(UCase(JG_Mode)), 1)
If Cal_Mode <> "J" Then Cal_Mode = "G"
  
'  ------------------------------------------------
'  Read calendar date (Assumed to be a valid date).
'  No error tests are done in this alpha version.

'   Strip all spaces from within date string.
    Date_String = Trim(UCase(Dd_Mmm_Yyyy_BCAD))
    Q = ""
For Pointer = 1 To Len(Date_String)
     Q1 = Mid(Date_String, Pointer, 1): If Q1 <> " " Then Q = Q & Q1
Next Pointer
     Date_String = Q
    
' Read the absolute value of the day of the month (1 to 31).
  Day = Trim(Abs(Val(Date_String)))

' Set (Pointer) to 1st character beyond the (Day) value.  This
' should be the 1st character in the 3 letter month name abbreviation.
  Pointer = InStr(1, Date_String, Day) + Len(Day)
   
' Now get the 3 letters of the month abbreviation and then update the (Pointer)
' to to aim at the following (Year) value.
  Month_Abbreviation = Mid(Date_String, Pointer, 3): Pointer = Pointer + 3
  
' Determine the number of the month corresponding to the three letter
' (Month_Abbreviation).  This abbreviation is simply the first three letters
' in the name of the month in English.  The returned value will range from
' 1=January to 12=December.  Zero is returned if no match is found
' for the abbreviation.  This test is NOT case sensitive.
  Month = Int(1 + ((InStr(1, UCase(Month_Abbreviations), _
          Month_Abbreviation) - 1) / 3))
  
' Read the absolute value of the year (1 to 9999).
  Year = Abs(Val(Mid(Date_String, Pointer, Len(Date_String))))
  
'  Check for BC or AD - If NOT a BC year, then assume AD.
If Right(Date_String, 2) = "BC" Then BCAD = "BC" Else BCAD = "AD"
If BCAD = "BC" Then Year = (1 - Year)

'  At this point the separate numerical date elements should be collected
'  and ready to be used for any subsequent calendar computations.
'  They are:  (Day) (Month) (Year) (BCAD) (Cal_Mode)
  
'     Now compute (JD) number according to the original Julian calendar.
  Q = Int((14 - Month) / 12)
 JD = Day + Int(367 * (Month - 2 + Q * 12) / 12) + _
      Int(1461 * (Year + 4800 - Q) / 4) - 32113
    
'  Apply Julian (JD) correction to convert to (JD) according to the Gregorian calendar
'  if indicated by (Cal_Mode).
If Cal_Mode = "G" Then JD = JD - (Int(3 * Int((Year + 100 - Q) / 100) / 4) - 2)
    
'  Return the final calendar day index (0 to 6) and then convert it into the
'  3 letter English abbreviation for the weekday.
   Day_Of_Week_For = (JD + 1) - 7 * Int((JD + 1) / 7)
  
'  If necessary, account for the case where JD is a negative value to
'  avoid a discontinuity in the weekday sequence.
If (Day_Of_Week_For < 0) Then Day_Of_Week_For = 6 + Day_Of_Week_For
 
'  Replace the numerical value of the weekday index with the corresponding
'  3 letter English abbreviation for the weekday.
   Day_Of_Week_For = Mid("SunMonTueWedThuFriSat", (3 * Day_Of_Week_For) + 1, 3)

End Function

Public Function Date_Corresponding_To(JD_Number, JG_Mode)
' Compute date string corresponding to given JD number and (JG_Mode).
' This is the opposite of the JD_Number_For() function.
'
' Version Alpha 0.001
' FUNCTION LEVEL 0 - No external function dependency.
'
' Any calendar dates returned will have the general format
' similar to "21 Mar 1988 AD"
' The year returned will always be in the BC/AD format.
'
' This function assumes that all input arguments are valid.
' NO error checking is done.
'
' Define the calendar date variables being computed
  Dim Month, Day, Year
  
' Define general working variables
  Dim JD, Cal_Mode, JG, Q, R, S, T, u, V

' Read in value of JD number for a date in the range
' Julian JD range = -1,930,711 to 5,373,557
' Gregorian JD range = -1,930,634 to 5,373,484
' These ranges correspond to calendar dates in the range
' from 1 Jan 9999 BC  to  31 Dec 9999 AD on the respective
' calendars as indicated.
  JD = Val(Trim(JD_Number))

'  Read the (Cal_Mode) from the (JG_Mode).  This determines the calendar,
'  Julian or Gregorian, to which the computed JD number applies.
'  Only the 1st character of (JG_Mode) is used. If that character is "J"
'  then Julian "J" mode is set, otherwise Gregorian "G" mode is set.
'  This test is NOT case sensitive.
   Cal_Mode = Left(Trim(UCase(JG_Mode)), 1)
If Cal_Mode <> "J" Then Cal_Mode = "G": JG = 1 Else JG = 0

'  At this point, the necessary variables have been collected and are
'  ready for use any the subsequent calendar computations.
'  They are (JD) and (JG_Mode).
'  ----------------------------

' Do auxiliary inverse JD computations
  Q = JG * Int((JD / 36524.25) - 51.12264)
  R = JD + JG + Q - Int(Q / 4)
  S = R + 1524
  T = Int((S / 365.25) - 0.3343)
  u = Int(T * 365.25)
  V = Int((S - u) / 30.61)

' Compute numerical values of the unknown date elements
  Month = (V - 1) + 12 * (V > 13.5)
  Day = S - u - Int(V * 30.61): Day = Trim(Day)
  Year = T - (Month < 2.5) - 4716
  
'  Now convert the numerical (Day), (Month) and (Year) values into
'  a date output string of the form "Dd Mmm Yyyy BC|AD" for output.

'  Express (Year) in BC/AD format
If Year <= 0 Then Year = Trim(1 - Year) & " BC" Else Year = Trim(Year) & " AD"
   
'  Replace the numerical value of the month with the corresponding 3 letter
'  English abbreviation for the month name.
   Month = Mid("JanFebMarAprMayJunJulAugSepOctNovDec", (3 * (Month - 1)) + 1, 3)

'  Construct the final finished output date string.
'  This same output is in the same format accepted as input by the other
'  related calendar modules, so this function's output can be use as another
'  function's input.
   Date_Corresponding_To = Day & " " & Month & " " & Year

End Function

Public Function Days_In_Month(Mmm_Yyyy_BCAD, JG_Mode)
' Compute the number of days in a given month and year
' on either the Julian or Gregorian calendars.
'
' Version Alpha 0.001
' FUNCTION LEVEL 0 - No external function dependency.
'
' This function assumes that all input arguments are valid.
' NO error checking is done.
'
'
'  Define the main function variables
   Dim Date_String, Cal_Mode As String
   Dim Day, Month, Year, BCAD, JD
   Dim Month_Abbreviation As String
   Dim Y, Leap

'  Define random work variables
   Dim Q, Q1
   Dim Pointer As Integer
    
'  Read the (Cal_Mode) from the (JG_Mode).  This determines the calendar,
'  Julian or Gregorian, to which the computed JD number applies.
'  Only the 1st character of (JG_Mode) is used. If that character is "J"
'  then Julian "J" mode is set, otherwise Gregorian "G" mode is set.
'  This test is NOT case sensitive.
   Cal_Mode = Left(Trim(UCase(JG_Mode)), 1)
If Cal_Mode <> "J" Then Cal_Mode = "G"
  
' ------------------------------------------------
' Read calendar date (Assumed to be a valid date).
' No error tests are done in this alpha version.

'   Strip all spaces from within date string.
    Date_String = Trim(UCase(Mmm_Yyyy_BCAD))
    Q = ""
For Pointer = 1 To Len(Date_String)
     Q1 = Mid(Date_String, Pointer, 1): If Q1 <> " " Then Q = Q & Q1
Next Pointer
     Date_String = Q
    
' Set to 1st day of month.
  Day = 1
    
' Now get the 3 letters of the month abbreviation and then update the (Pointer)
' to to aim at the following (Year) value.
  Month_Abbreviation = Mid(Date_String, 1, 3)
  
' Determine the number of the month corresponding to the three letter
' (Month_Abbreviation).  This abbreviation is simply the first three letters
' in the name of the month in English.  The returned value will range from
' 1=January to 12=December.  Zero is returned if no match is found
' for the abbreviation.  This test is NOT case sensitive.
  Month = Int(1 + ((InStr(1, UCase("JanFebMarAprMayJunJulAugSepOctNovDec"), _
          Month_Abbreviation) - 1) / 3))
  
' Read the absolute value of the year (1 to 9999).
  Year = Abs(Val(Mid(Date_String, 4, Len(Date_String))))
  
'  Check for BC or AD - If NOT a BC year, then assume AD.
If Right(Date_String, 2) = "BC" Then BCAD = "BC" Else BCAD = "AD"
If BCAD = "BC" Then Year = (1 - Year)

' At this point the separate numerical date elements should be collected
' and ready to be used for any subsequent calendar computations. They are:
' (Day: 1)  (Month: 1 to 12)  (Year: 1 to 9999 BC|AD)  (Cal_Mode: J|G)
' --------------------------------------------------------------------
'
'  Construct date string for 1st day of given month and determine if leap year
'  and adjust accordingly.
If Year <= 0 Then Y = Trim(1 - Year) & " BC" Else Y = Trim(Year) & " AD"
   GoSub TEST_LEAP_YEAR
   If Q = True Then Leap = 1 Else Leap = 0
   Days_In_Month = Mid("312831303130313130313031", 2 * (Month - 1) + 1, 2)
If Month = 2 Then Days_In_Month = Days_In_Month + Leap
Exit Function

'  Subroutine to check if a leap year - This is to make this a level zero function and
'  independent of external functions.
'
'  Determine if (Year) is leap on Julian calendar if "J" mode indicated,
'  otherwise, ignore this operation and solve for Gregorian calendar.
TEST_LEAP_YEAR:
   If Cal_Mode = "J" Then Q = ((Year Mod 4) = 0): GoTo LEAP_TEST_DONE
'  Determine if (Year) is leap on the Gregorian calendar.
If (Year Mod 100) = 0 Then Q = ((Year Mod 400) = 0): GoTo LEAP_TEST_DONE
   Q = ((Year Mod 4) = 0)
LEAP_TEST_DONE:
   Return
   
End Function

' ///////////////////////////////////
' \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
'
' LEVEL 1 FUNCTIONS

Public Function Days_Between_Dates(Dd1_Mmm1_Yyyy1_BCAD1, Dd2_Mmm2_Yyyy2_BCAD2, JG_Mode)
' This function returns the absolute difference in days between any
' two dates on the same calendar system.  It works for both the Julian
' and Gregorian calendars.
'
' Version Alpha 0.001
' FUNCTION LEVEL 1 - Requires one LEVEL 0 function.
' REQUIRED LEVEL 0 FUNCTION: JD_Number_For()
'
' This function assumes that all input arguments are valid.
' NO error checking is done.
'
' Define main function variables.
  Dim Date1, Date2, Cal_Mode, JD1, JD2

' Read in the two date arguments.
  Date1 = Trim(UCase(Dd1_Mmm1_Yyyy1_BCAD1))
  Date2 = Trim(UCase(Dd2_Mmm2_Yyyy2_BCAD2))

'  Read the (JG_Mode) as (Cal_Mode) internally.  This determines the
'  calendar, Julian or Gregorian, to which the computed JD number applies.
'  Only the 1st character of (JG_Mode) is used. If that character is "J"
'  then Julian "J" mode is set, otherwise Gregorian "G" mode is set.
'  This test is NOT case sensitive.
   Cal_Mode = Left(Trim(UCase(JG_Mode)), 1)
If Cal_Mode <> "J" Then Cal_Mode = "G"

' Compute the JD numbers for each of the two dates.
  JD1 = JD_Number_For(Date1, Cal_Mode)
  JD2 = JD_Number_For(Date2, Cal_Mode)

' Now compute the absolute difference in days between the dates.
  Days_Between_Dates = Abs(JD2 - JD1)
  
End Function

Public Function Calendar_Day_Number_For(Dd_Mmm_Yyyy_BCAD, JG_Mode)
' Compute the calendar day number within the given year for the
' given date and calendar mode.
'
' Version Alpha 0.001
' FUNCTION LEVEL 1 - Requires one LEVEL 0 function.
' REQUIRED LEVEL 0 FUNCTION: JD_Number_For()
'
' This function assumes that all input arguments are valid.
' NO error checking is done.
'
'
'  Define the main function variables
   Dim Date_String, Cal_Mode As String
   Dim Day, Month, Year, BCAD, JD
   Dim Month_Abbreviation As String
   Dim JD_31DecLast, JD_For_Date, Y, CD_Number

'  Define random work variables
   Dim Q, Q1
   Dim Pointer As Integer
    
'  Read the (Cal_Mode) from the (JG_Mode).  This determines the calendar,
'  Julian or Gregorian, to which the computed JD number applies.
'  Only the 1st character of (JG_Mode) is used. If that character is "J"
'  then Julian "J" mode is set, otherwise Gregorian "G" mode is set.
'  This test is NOT case sensitive.
   Cal_Mode = Left(Trim(UCase(JG_Mode)), 1)
If Cal_Mode <> "J" Then Cal_Mode = "G"
  
' ------------------------------------------------
' Read calendar date (Assumed to be a valid date).
' No error tests are done in this alpha version.

'   Strip all spaces from within date string.
    Date_String = Trim(UCase(Dd_Mmm_Yyyy_BCAD))
    Q = ""
For Pointer = 1 To Len(Date_String)
     Q1 = Mid(Date_String, Pointer, 1): If Q1 <> " " Then Q = Q & Q1
Next Pointer
     Date_String = Q
    
' Read the absolute value of the day of the month (1 to 31).
  Day = Trim(Abs(Val(Date_String)))

' Set (Pointer) to 1st character beyond the (Day) value.  This
' should be the 1st character in the 3 letter month name abbreviation.
  Pointer = InStr(1, Date_String, Day) + Len(Day)
   
' Now get the 3 letters of the month abbreviation and then update the (Pointer)
' to to aim at the following (Year) value.
  Month_Abbreviation = Mid(Date_String, Pointer, 3): Pointer = Pointer + 3
  
' Determine the number of the month corresponding to the three letter
' (Month_Abbreviation).  This abbreviation is simply the first three letters
' in the name of the month in English.  The returned value will range from
' 1=January to 12=December.  Zero is returned if no match is found
' for the abbreviation.  This test is NOT case sensitive.
  Month = Int(1 + ((InStr(1, UCase("JanFebMarAprMayJunJulAugSepOctNovDec"), _
          Month_Abbreviation) - 1) / 3))
  
' Read the absolute value of the year (1 to 9999).
  Year = Abs(Val(Mid(Date_String, Pointer, Len(Date_String))))
  
'  Check for BC or AD - If NOT a BC year, then assume AD.
If Right(Date_String, 2) = "BC" Then BCAD = "BC" Else BCAD = "AD"
If BCAD = "BC" Then Year = (1 - Year)

' At this point the separate numerical date elements should be collected
' and ready to be used for any subsequent calendar computations. They are:
' (Day: 1 to 31)  (Month: 1 to 12)  (Year: 1 to 9999 BC|AD)  (Cal_Mode: J|G)

'  Compute JD number for 31 December of the previous year.
   Y = Year - 1
If Y <= 0 Then Y = Trim(1 - Y) & " BC" Else Y = Trim(Y) & " AD"
   JD_31DecLast = JD_Number_For("31 Dec " & Y, Cal_Mode)
  
' Compute JD number for given date.
  Y = Year: If Y <= 0 Then Y = Trim(1 - Y) & " BC" Else Y = Trim(Y) & " AD"
  JD_For_Date = JD_Number_For(Day & Month_Abbreviation & Y, Cal_Mode)

' Now compute the calendar day number within the year for the given date.
  Calendar_Day_Number_For = (JD_For_Date - JD_31DecLast)
  
End Function

